@@ -4,14 +4,13 @@ from typing import Dict, List, Optional, Tuple, Union
 
 import numpy as np
 import pandas as pd
-from desdeo_problem.Problem import MOProblem
-from desdeo_tools.interaction.request import BaseRequest, SimplePlotRequest
-from desdeo_tools.scalarization.ASF import AugmentedGuessASF, MaxOfTwoASF, PointMethodASF, SimpleASF, StomASF
-from desdeo_tools.scalarization.Scalarizer import Scalarizer
-from desdeo_tools.solver.ScalarSolver import ScalarMethod, ScalarMinimizer
-
 from desdeo_mcdm.interactive.InteractiveMethod import InteractiveMethod
 from desdeo_mcdm.utilities.solvers import payoff_table_method
+from desdeo_problem.problem import DiscreteDataProblem, MOProblem
+from desdeo_tools.interaction.request import BaseRequest, SimplePlotRequest
+from desdeo_tools.scalarization.ASF import AugmentedGuessASF, MaxOfTwoASF, PointMethodASF, SimpleASF, StomASF
+from desdeo_tools.scalarization.Scalarizer import DiscreteScalarizer, Scalarizer
+from desdeo_tools.solver.ScalarSolver import DiscreteMinimizer, ScalarMethod, ScalarMinimizer
 
 
 class NimbusException(Exception):
@@ -42,7 +41,7 @@ class NimbusClassificationRequest(BaseRequest):
             "\n\t4. values which may be impaired until some upper bound is reached '>='"
             "\n\t5. values which are free to change '0'"
             "\nProvide the aspiration levels and upper bounds as a vector. For categories 1, 3, and 5,"
-            "the value in the vector at the objective's position is ignored. Suppy also the number of maximum"
+            "the value in the vector at the objective's position is ignored. Supply also the number of maximum"
             "solutions to be generated."
         )
 
@@ -88,7 +87,7 @@ class NimbusClassificationRequest(BaseRequest):
         is_valid_cls = map(lambda x: x in self._valid_classifications, response["classifications"],)
 
         if not all(list(is_valid_cls)):
-            raise NimbusException(f"Invalid classificaiton found in {response['classifications']}")
+            raise NimbusException(f"Invalid classification found in {response['classifications']}")
 
         # check the levels
         if len(np.array(response["levels"]).squeeze()) != self._method._problem.n_of_objectives:
@@ -143,7 +142,7 @@ class NimbusSaveRequest(BaseRequest):
     ):
         msg = (
             "Please specify which solutions shown you would like to save for later viewing. Supply the "
-            "indices of such solutions as a list, or supply an empty list if none of the shown soulutions "
+            "indices of such solutions as a list, or supply an empty list if none of the shown solutions "
             "should be saved."
         )
         content = {
@@ -169,7 +168,7 @@ class NimbusSaveRequest(BaseRequest):
         if "indices" not in response:
             raise NimbusException("'indices' entry missing")
 
-        if not response["indices"]:
+        if len(response["indices"]) == 0:
             # nothing to save, continue to next state
             return
 
@@ -204,7 +203,7 @@ class NimbusIntermediateSolutionsRequest(BaseRequest):
         self, solution_vectors: List[np.ndarray], objective_vectors: List[np.ndarray],
     ):
         msg = (
-            "Would you like to see intermediate solutions between two previusly computed solutions? "
+            "Would you like to see intermediate solutions between two previously computed solutions? "
             "If so, please supply two indices corresponding to the solutions."
         )
 
@@ -240,13 +239,13 @@ class NimbusIntermediateSolutionsRequest(BaseRequest):
         if response["number_of_desired_solutions"] < 0:
             raise NimbusException(f"Invalid number of desired solutions {response['number_of_desired_solutions']}.")
 
-        if not response["indices"] and response["number_of_desired_solutions"] > 0:
-            raise NimbusException("Indices supplied yet number of desired soutions is greater than zero.")
+        if len(response["indices"]) > 0 and response["number_of_desired_solutions"] == 0:
+            raise NimbusException("Indices supplied yet number of desired solutions is zero.")
 
-        if response["indices"] and response["number_of_desired_solutions"] == 0:
-            raise NimbusException("Indices not supplied yet number of desired soutions is zero.")
+        if len(response["indices"]) == 0 and response["number_of_desired_solutions"] > 0:
+            raise NimbusException("Indices not supplied yet number of desired solutions is greater than zero.")
 
-        if not response["indices"]:
+        if len(response["indices"]) == 0:
             return
 
         if np.max(response["indices"]) >= len(self.content["objectives"]) or np.min(response["indices"]) < 0:
@@ -268,7 +267,7 @@ class NimbusMostPreferredRequest(BaseRequest):
 
     Note:
         The objective vector at position 'i' in `objective_vectors` should correspond to the decision variables at
-        position 'i' in `solution_vectors`. Only the two first entries in each of the lists is relevant. The preferred
+        position 'i' in `solution_vectors`. Only the two first entries in each of the lists are relevant. The preferred
         solution will be selected from `objective_vectors`.
 
     """
@@ -357,11 +356,17 @@ class NIMBUS(InteractiveMethod):
 
     Args:
         problem (MOProblem): The problem to be solved.
-        scalar_method (Optional[ScalarMethod], optional): The method used to solve
-            the various ASF minimization problems present in the method. Defaults to None.
+        scalar_method (Optional[Union[ScalarMethod, str]], optional): The method used to solve
+            the various ASF minimization problems present in the method. Defaults to 'scipy_de' (differential evolution).
+        starting_point(Optional[np.ndarray], optional): The initial solution (objectives) to start classification from.
+            If None, a neutral starting point will be computed. 
+    Note:
+        When a starting point is supplied, decision variables of that point will be approximated to be the variables of the
+        solution closest to the starting point. In other words, the decision variables associated to the initial point may be
+        inaccurate!
     """
 
-    def __init__(self, problem: MOProblem, scalar_method: Optional[ScalarMethod] = None):
+    def __init__(self, problem: Union[MOProblem, DiscreteDataProblem], scalar_method: Optional[Union[ScalarMethod, str]] = "scipy_de", starting_point: Optional[np.ndarray] = None):
         # check if ideal and nadir are defined
         if problem.ideal is None or problem.nadir is None:
             # TODO: use same method as defined in scalar_method
@@ -374,30 +379,65 @@ class NIMBUS(InteractiveMethod):
 
         self._scalar_method = scalar_method
 
-        # generate Pareto optimal starting point
-        asf = SimpleASF(np.ones(self._ideal.shape))
-        scalarizer = Scalarizer(
-            lambda x: problem.evaluate(x).objectives,
-            asf,
-            scalarizer_args={"reference_point": np.atleast_2d(self._ideal)},
-        )
+        # check starting point if given
+        if starting_point is not None:
+            if np.squeeze(starting_point).shape != np.squeeze(self._ideal.shape):
+                raise NimbusException(
+                    f"The given starting point {starting_point} has mismatching dimensions {starting_point.shape}."
+                    )
+
+        if isinstance(problem, MOProblem):
+            # Change me to be the right ASF
+            # if starting point was given, use that as the reference point
+            if starting_point is None:
+                reference_point = (self._ideal + self._nadir) / 2
+            else:
+                reference_point = starting_point
+
+            asf = PointMethodASF(self._nadir, self._ideal)
+            scalarizer = Scalarizer(
+                lambda x: problem.evaluate(x).objectives,
+                asf,
+                scalarizer_args={"reference_point": reference_point},
+            )
+
+            if problem.n_of_constraints > 0:
+                _con_eval = lambda x: problem.evaluate(x).constraints.squeeze()
+            else:
+                _con_eval = None
+
+            solver = ScalarMinimizer(
+                scalarizer, problem.get_variable_bounds(), constraint_evaluator=_con_eval, method=self._scalar_method,
+            )
+            # TODO: fix tools to check for scipy methods in general and delete me!
+            solver._use_scipy = True
+
+            res = solver.minimize(problem.get_variable_upper_bounds() / 2)
+
+            if res["success"]:
+                self._current_solution = res["x"]
+                self._current_objectives = problem.evaluate(self._current_solution).objectives.squeeze()
+            else:
+                raise NimbusException("Could not solve the initial ASF.")
+
+        elif isinstance(problem, DiscreteDataProblem):
+            # discrete case
+            if starting_point is None:
+                reference_point = (self._ideal + self._nadir) / 2
+            else:
+                reference_point = starting_point
+
+            asf = PointMethodASF(self._nadir, self._ideal)
+            scalarizer = DiscreteScalarizer(asf, scalarizer_args={"reference_point": reference_point})
+            solver = DiscreteMinimizer(scalarizer)
+
+            res = solver.minimize(problem.objectives)
+            self._current_solution = problem.decision_variables[res["x"]]
+            self._current_objectives = problem.objectives[res["x"]]
 
-        if problem.n_of_constraints > 0:
-            _con_eval = lambda x: problem.evaluate(x).constraints.squeeze()
         else:
-            _con_eval = None
-
-        solver = ScalarMinimizer(
-            scalarizer, problem.get_variable_bounds(), constraint_evaluator=_con_eval, method=self._scalar_method,
-        )
-        # TODO: fix tools to check for scipy methods in general and delete me!
-        solver._use_scipy = True
-
-        res = solver.minimize(problem.get_variable_upper_bounds() / 2)
-
-        if res["success"]:
-            self._current_solution = res["x"]
-            self._current_objectives = problem.evaluate(self._current_solution).objectives.squeeze()
+            # unsupported problem type
+            raise NimbusException(f"Unsupported problem type {type(problem)}.")
 
         self._archive_solutions = []
         self._archive_objectives = []
@@ -430,14 +470,22 @@ class NIMBUS(InteractiveMethod):
         Returns:
             SimplePlotRequest: A plot request to create a visualization.
         """
-        dimensions_data = pd.DataFrame(
-            index=["minimize", "ideal", "nadir"], columns=self._problem.get_objective_names(),
-        )
-        dimensions_data.loc["minimize"] = self._problem._max_multiplier
-        dimensions_data.loc["ideal"] = self._ideal
-        dimensions_data.loc["nadir"] = self._nadir
+        if isinstance(self._problem, MOProblem):
+            dimensions_data = pd.DataFrame(
+                index=["minimize", "ideal", "nadir"], columns=self._problem.get_objective_names(),
+            )
+            dimensions_data.loc["minimize"] = self._problem._max_multiplier
+            dimensions_data.loc["ideal"] = self._ideal
+            dimensions_data.loc["nadir"] = self._nadir
 
-        data = pd.DataFrame(objectives, columns=self._problem.get_objective_names())
+            data = pd.DataFrame(objectives, columns=self._problem.get_objective_names())
+        else:
+            dimensions_data = pd.DataFrame(index=["minimize", "ideal", "nadir"], columns=self._problem.objective_names,)
+            dimensions_data.loc["minimize"] = [1 for _ in self._problem.objective_names]
+            dimensions_data.loc["ideal"] = self._ideal
+            dimensions_data.loc["nadir"] = self._nadir
+
+            data = pd.DataFrame(objectives, columns=self._problem.objective_names)
 
         plot_request = SimplePlotRequest(data=data, dimensions_data=dimensions_data, message=msg,)
 
@@ -449,7 +497,7 @@ class NIMBUS(InteractiveMethod):
         """Handles a classification request.
         
         Args:
-            request (NimbusClassificationReuest): A classification request with the
+            request (NimbusClassificationRequest): A classification request with the
                 response attribute set.
         
         Returns:
@@ -514,7 +562,7 @@ class NIMBUS(InteractiveMethod):
             decision maker wishes to see intermediate points, the latter otherwise. Also a plot request is
             returned with the solutions available in it.
         """
-        if request.response["indices"]:
+        if len(request.response["indices"]) > 0:
             return self.compute_intermediate_solutions(
                 np.array(request.content["solutions"])[request.response["indices"]],
                 int(request.response["number_of_desired_solutions"]),
@@ -527,7 +575,7 @@ class NIMBUS(InteractiveMethod):
     def handle_most_preferred_request(
         self, request: NimbusMostPreferredRequest
     ) -> Tuple[Union[NimbusClassificationRequest, NimbusStopRequest], SimplePlotRequest]:
-        """Handles a preferres solution request.
+        """Handles a preferred solution request.
         
         Args:
             request (NimbusMostPreferredRequest): A NIMBUS preferred solution request with the
@@ -535,7 +583,7 @@ class NIMBUS(InteractiveMethod):
         
         Returns:
             Tuple[Union[NimbusClassificationRequest, NimbusStopRequest], SimplePlotRequest]:
-            Return a classificaiton request if the decision maker wishes to continue. If the
+            Return a classification request if the decision maker wishes to continue. If the
             decision maker wishes to stop, return a stop request. Also return a plot 
             request with all the solutions saved so far.
         """
@@ -593,10 +641,10 @@ class NIMBUS(InteractiveMethod):
     def compute_intermediate_solutions(
         self, solutions: np.ndarray, n_desired: int,
     ) -> Tuple[NimbusSaveRequest, SimplePlotRequest]:
-        """Computs intermediate solution between two solutions computed earlier.
+        """Computes intermediate solution between two solutions computed earlier.
 
         Args:
-            solutions (np.ndarray): The solutions between which the intermediat solutions should
+            solutions (np.ndarray): The solutions between which the intermediate solutions should
                 be computed.
             n_desired (int): The number of intermediate solutions desired.
         
@@ -604,7 +652,7 @@ class NIMBUS(InteractiveMethod):
             NimbusException
         
         Returns:
-            Tuple[NimbusSaveRequest, SimplePlotRequest]: A save request with the compured intermediate
+            Tuple[NimbusSaveRequest, SimplePlotRequest]: A save request with the computed intermediate
             points, and a plot request to visualize said points.
         """
         # vector between the two solutions
@@ -624,24 +672,41 @@ class NIMBUS(InteractiveMethod):
         asf = PointMethodASF(self._nadir, self._ideal)
 
         for i in range(n_desired):
-            scalarizer = Scalarizer(
-                lambda x: self._problem.evaluate(x).objectives,
-                asf,
-                scalarizer_args={"reference_point": self._problem.evaluate(intermediate_points[i]).objectives},
-            )
+            if isinstance(self._problem, MOProblem):
+                scalarizer = Scalarizer(
+                    lambda x: self._problem.evaluate(x).objectives,
+                    asf,
+                    scalarizer_args={"reference_point": self._problem.evaluate(intermediate_points[i]).objectives},
+                )
+
+                if self._problem.n_of_constraints > 0:
+                    cons = lambda x: self._problem.evaluate(x).constraints.squeeze()
+                else:
+                    cons = None
+
+                solver = ScalarMinimizer(
+                    scalarizer, self._problem.get_variable_bounds(), cons, method=self._scalar_method,
+                )
+
+                res = solver.minimize(self._current_solution)
+                intermediate_solutions[i] = res["x"]
+                intermediate_objectives[i] = self._problem.evaluate(res["x"]).objectives
 
-            if self._problem.n_of_constraints > 0:
-                cons = lambda x: self._problem.evaluate(x).constraints.squeeze()
             else:
-                cons = None
+                # discrete case
+                scalarizer = DiscreteScalarizer(
+                    asf,
+                    scalarizer_args={
+                        "reference_point": self._problem.objectives[self._problem.find_closest(intermediate_points[i])]
+                    },
+                )
+                solver = DiscreteMinimizer(scalarizer)
 
-            solver = ScalarMinimizer(scalarizer, self._problem.get_variable_bounds(), cons, method=self._scalar_method,)
+                res = solver.minimize(self._problem.objectives)
+                intermediate_solutions[i] = self._problem.decision_variables[res["x"]]
+                intermediate_objectives[i] = self._problem.objectives[res["x"]]
 
-            res = solver.minimize(self._current_solution)
-            intermediate_solutions[i] = res["x"]
-            intermediate_objectives[i] = self._problem.evaluate(res["x"]).objectives
-
-        # create appropiate requests
+        # create appropriate requests
         save_request = NimbusSaveRequest(list(intermediate_solutions), list(intermediate_objectives))
 
         msg = "Computed intermediate solutions"
@@ -695,7 +760,7 @@ class NIMBUS(InteractiveMethod):
         impaire_until_inds: np.ndarray,
         free_inds: np.ndarray,
     ) -> Tuple[NimbusSaveRequest, SimplePlotRequest]:
-        """Calcualtes new solutions based on classifications supplied by the decision maker by
+        """Calculates new solutions based on classifications supplied by the decision maker by
             solving ASF problems.
         
         Args:
@@ -709,7 +774,7 @@ class NIMBUS(InteractiveMethod):
             free_inds (np.ndarray): Indices of objectives which may change freely.
         
         Returns:
-            Tuple[NimbusSaveRequest, SimplePlotRequest]: A save request with the newly computed soutions, and 
+            Tuple[NimbusSaveRequest, SimplePlotRequest]: A save request with the newly computed solutions, and 
             a plot request to visualize said solutions.
         """
         results = []
@@ -717,39 +782,70 @@ class NIMBUS(InteractiveMethod):
         # always computed
         asf_1 = MaxOfTwoASF(self._nadir, self._ideal, improve_inds, improve_until_inds)
 
-        def cons_1(
-            x: np.ndarray,
-            f_current: np.ndarray = self._current_objectives,
-            levels: np.ndarray = levels,
-            improve_until_inds: np.ndarray = improve_until_inds,
-            improve_inds: np.ndarray = improve_inds,
-            impaire_until_inds: np.ndarray = impaire_until_inds,
-        ):
-            f = self._problem.evaluate(x).objectives.squeeze()
+        if isinstance(self._problem, MOProblem):
 
-            res_1 = f_current[improve_inds] - f[improve_inds]
-            res_2 = f_current[improve_until_inds] - f[improve_until_inds]
-            res_3 = levels[impaire_until_inds] - f_current[impaire_until_inds]
+            def cons_1(
+                x: np.ndarray,
+                f_current: np.ndarray = self._current_objectives,
+                levels: np.ndarray = levels,
+                improve_until_inds: np.ndarray = improve_until_inds,
+                improve_inds: np.ndarray = improve_inds,
+                impaire_until_inds: np.ndarray = impaire_until_inds,
+                acceptable_inds: np.ndarray = acceptable_inds,
+            ):
+                f = self._problem.evaluate(x).objectives.squeeze()
 
-            res = np.hstack((res_1, res_2, res_3))
+                res_1 = f_current[improve_inds] - f[improve_inds]
+                res_2 = f_current[improve_until_inds] - f[improve_until_inds]
+                res_3 = f_current[acceptable_inds] - f[acceptable_inds]
+                res_4 = levels[impaire_until_inds] - f[impaire_until_inds]
 
-            if self._problem.n_of_constraints > 0:
-                res_prob = self._problem.evaluate(x).constraints.squeeze()
+                res = np.hstack((res_1, res_2, res_3, res_4))
 
-                return np.hstack((res_prob, res))
+                if self._problem.n_of_constraints > 0:
+                    res_prob = self._problem.evaluate(x).constraints.squeeze()
 
-            else:
-                return res
+                    return np.hstack((res_prob, res))
 
-        scalarizer_1 = Scalarizer(
-            lambda x: self._problem.evaluate(x).objectives, asf_1, scalarizer_args={"reference_point": levels},
-        )
+                else:
+                    return res
 
-        solver_1 = ScalarMinimizer(
-            scalarizer_1, self._problem.get_variable_bounds(), cons_1, method=self._scalar_method,
-        )
+            scalarizer_1 = Scalarizer(
+                lambda x: self._problem.evaluate(x).objectives, asf_1, scalarizer_args={"reference_point": levels},
+            )
+
+            solver_1 = ScalarMinimizer(
+                scalarizer_1, self._problem.get_variable_bounds(), cons_1, method=self._scalar_method,
+            )
+
+            res_1 = solver_1.minimize(self._current_solution)
+
+        else:
+            # discrete case
+            def cons_1(
+                objective_vectors: np.ndarray,
+                f_current: np.ndarray = self._current_objectives,
+                levels: np.ndarray = levels,
+                improve_until_inds: np.ndarray = improve_until_inds,
+                improve_inds: np.ndarray = improve_inds,
+                impaire_until_inds: np.ndarray = impaire_until_inds,
+                acceptable_inds: np.ndarray = acceptable_inds,
+            ):
+                res_1 = f_current[improve_inds] - objective_vectors[:, improve_inds]
+                res_2 = f_current[improve_until_inds] - objective_vectors[:, improve_until_inds]
+                res_3 = f_current[acceptable_inds] - objective_vectors[:, acceptable_inds]
+                res_4 = levels[impaire_until_inds] - objective_vectors[:, impaire_until_inds]
+
+                res = np.hstack((res_1, res_2, res_3, res_4)) >= 0
+
+                return np.all(res, axis=1)
+
+            scalarizer_1 = DiscreteScalarizer(asf_1, scalarizer_args={"reference_point": levels})
+
+            solver_1 = DiscreteMinimizer(scalarizer_1, cons_1)
+
+            res_1 = solver_1.minimize(self._problem.objectives)
 
-        res_1 = solver_1.minimize(self._current_solution)
         results.append(res_1)
 
         if number_of_solutions > 1:
@@ -765,55 +861,92 @@ class NIMBUS(InteractiveMethod):
             asf_2 = StomASF(self._ideal)
 
             # cons_2 can be used in the rest of the ASF scalarizations, it's not a bug!
-            if self._problem.n_of_constraints > 0:
+            if isinstance(self._problem, MOProblem) and self._problem.n_of_constraints > 0:
                 cons_2 = lambda x: self._problem.evaluate(x).constraints.squeeze()
             else:
                 cons_2 = None
 
-            scalarizer_2 = Scalarizer(
-                lambda x: self._problem.evaluate(x).objectives, asf_2, scalarizer_args={"reference_point": z_bar},
-            )
+            if isinstance(self._problem, MOProblem):
+                scalarizer_2 = Scalarizer(
+                    lambda x: self._problem.evaluate(x).objectives, asf_2, scalarizer_args={"reference_point": z_bar},
+                )
 
-            solver_2 = ScalarMinimizer(
-                scalarizer_2, self._problem.get_variable_bounds(), cons_2, method=self._scalar_method,
-            )
+                solver_2 = ScalarMinimizer(
+                    scalarizer_2, self._problem.get_variable_bounds(), cons_2, method=self._scalar_method,
+                )
+
+                res_2 = solver_2.minimize(self._current_solution)
+
+            else:
+                # discrete case
+                scalarizer_2 = DiscreteScalarizer(asf_2, scalarizer_args={"reference_point": z_bar})
+
+                solver_2 = DiscreteMinimizer(scalarizer_2, cons_2)
+
+                res_2 = solver_2.minimize(self._problem.objectives)
 
-            res_2 = solver_2.minimize(self._current_solution)
             results.append(res_2)
 
         if number_of_solutions > 2:
             # asf 3
             asf_3 = PointMethodASF(self._nadir, self._ideal)
 
-            scalarizer_3 = Scalarizer(
-                lambda x: self._problem.evaluate(x).objectives, asf_3, scalarizer_args={"reference_point": z_bar},
-            )
+            if isinstance(self._problem, MOProblem):
+                scalarizer_3 = Scalarizer(
+                    lambda x: self._problem.evaluate(x).objectives, asf_3, scalarizer_args={"reference_point": z_bar},
+                )
 
-            solver_3 = ScalarMinimizer(
-                scalarizer_3, self._problem.get_variable_bounds(), cons_2, method=self._scalar_method,
-            )
+                solver_3 = ScalarMinimizer(
+                    scalarizer_3, self._problem.get_variable_bounds(), cons_2, method=self._scalar_method,
+                )
+
+                res_3 = solver_3.minimize(self._current_solution)
+
+            else:
+                # discrete case
+                scalarizer_3 = DiscreteScalarizer(asf_3, scalarizer_args={"reference_point": z_bar})
+
+                solver_3 = DiscreteMinimizer(scalarizer_3, cons_2)
+
+                res_3 = solver_3.minimize(self._problem.objectives)
 
-            res_3 = solver_3.minimize(self._current_solution)
             results.append(res_3)
 
         if number_of_solutions > 3:
             # asf 4
             asf_4 = AugmentedGuessASF(self._nadir, self._ideal, free_inds)
 
-            scalarizer_4 = Scalarizer(
-                lambda x: self._problem.evaluate(x).objectives, asf_4, scalarizer_args={"reference_point": z_bar},
-            )
+            if isinstance(self._problem, MOProblem):
 
-            solver_4 = ScalarMinimizer(
-                scalarizer_4, self._problem.get_variable_bounds(), cons_2, method=self._scalar_method,
-            )
+                scalarizer_4 = Scalarizer(
+                    lambda x: self._problem.evaluate(x).objectives, asf_4, scalarizer_args={"reference_point": z_bar},
+                )
+
+                solver_4 = ScalarMinimizer(
+                    scalarizer_4, self._problem.get_variable_bounds(), cons_2, method=self._scalar_method,
+                )
+
+                res_4 = solver_4.minimize(self._current_solution)
+
+            else:
+                # discrete case
+                scalarizer_4 = DiscreteScalarizer(asf_4, scalarizer_args={"reference_point": z_bar})
+
+                solver_4 = DiscreteMinimizer(scalarizer_4, cons_2)
+
+                res_4 = solver_4.minimize(self._problem.objectives)
 
-            res_4 = solver_4.minimize(self._current_solution)
             results.append(res_4)
 
         # create the save request
-        solutions = [res["x"] for res in results]
-        objectives = [self._problem.evaluate(x).objectives.squeeze() for x in solutions]
+        if isinstance(self._problem, MOProblem):
+            solutions = [res["x"] for res in results]
+            objectives = [self._problem.evaluate(x).objectives.squeeze() for x in solutions]
+
+        else:
+            # discrete case
+            solutions = [self._problem.decision_variables[res["x"]] for res in results]
+            objectives = [self._problem.objectives[res["x"]] for res in results]
 
         save_request = NimbusSaveRequest(solutions, objectives)
 
@@ -857,14 +990,14 @@ class NIMBUS(InteractiveMethod):
         Union[NimbusClassificationRequest, NimbusSaveRequest, NimbusIntermediateSolutionsRequest,],
         Union[SimplePlotRequest, None],
     ]:
-        """Implements a finite state machine to iterate over the different steps defined in Synchronous NIMBUS based on a supllied request.
+        """Implements a finite state machine to iterate over the different steps defined in Synchronous NIMBUS based on a supplied request.
         
         Args:
             request (Union[NimbusClassificationRequest,NimbusSaveRequest,NimbusIntermediateSolutionsRequest,NimbusMostPreferredRequest,NimbusStopRequest,]):
                 A request based on the next step in the NIMBUS algorithm is taken.
         
         Raises:
-            NimbusException: If a wrong type of request is supllied based on the current state NIMBUS is in.
+            NimbusException: If a wrong type of request is supplied based on the current state NIMBUS is in.
         
         Returns:
             Tuple[Union[NimbusClassificationRequest,NimbusSaveRequest,NimbusIntermediateSolutionsRequest,],Union[SimplePlotRequest, None],]:
@@ -927,9 +1060,9 @@ class NIMBUS(InteractiveMethod):
 
 
 if __name__ == "__main__":
-    from desdeo_problem.Objective import _ScalarObjective
-    from desdeo_problem.Variable import variable_builder
-    from desdeo_problem.Constraint import ScalarConstraint
+    from desdeo_problem.problem.Objective import _ScalarObjective
+    from desdeo_problem.problem.Variable import variable_builder
+    from desdeo_problem.problem.Constraint import ScalarConstraint
 
     # create the problem
     def f_1(x):
@@ -990,3 +1123,102 @@ if __name__ == "__main__":
     res_3.response = response_pref
 
     res_4 = method.iterate(res_3)
+
+    """
+    import matplotlib.pyplot as plt
+    from desdeo_problem.problem import _ScalarObjective, variable_builder
+
+    def f_1(xs: np.ndarray):
+        xs = np.atleast_2d(xs)
+        xs_plusone = np.roll(xs, 1, axis=1)
+        return np.sum(-10 * np.exp(-0.2 * np.sqrt(xs[:, :-1] ** 2 + xs_plusone[:, :-1] ** 2)), axis=1)
+
+    def f_2(xs: np.ndarray):
+        xs = np.atleast_2d(xs)
+        return np.sum(np.abs(xs) ** 0.8 + 5 * np.sin(xs ** 3), axis=1)
+
+    varsl = variable_builder(
+        ["x_1", "x_2", "x_3"], initial_values=[0, 0, 0], lower_bounds=[-5, -5, -5], upper_bounds=[5, 5, 5],
+    )
+
+    f1 = _ScalarObjective(name="f1", evaluator=f_1)
+    f2 = _ScalarObjective(name="f2", evaluator=f_2)
+
+    # For AI_DM ranges are always defined as IDEAL, NADIR.
+    f1_range = [-20, -14]
+    f2_range = [-14, 0.5]
+    x1 = np.linspace(min(f1_range), max(f1_range), 1000)
+    x2 = np.linspace(min(f2_range), max(f2_range), 1000)
+    y = np.linspace(0, 0, 1000)
+
+    problem = MOProblem(variables=varsl, objectives=[f1, f2], ideal=np.array([-20, -12]), nadir=np.array([-14, 0.5]))
+
+    from desdeo_mcdm.interactive.NIMBUS import NIMBUS
+    from scipy.optimize import differential_evolution, minimize
+
+    scalar_method = ScalarMethod(
+        lambda x, _, **y: differential_evolution(x, **y), use_scipy=True, method_args={"polish": True, "disp": True}
+    )
+
+    method = NIMBUS(problem, scalar_method)
+
+    classification_request, plot_request = method.start()
+
+    # print(classification_request.content.keys())
+    # print(classification_request.content["message"])
+
+    print(classification_request.content["objective_values"])
+
+    # Plotting F1!
+    plt.scatter(x1, y, label="Range")
+    plt.scatter(-15, 0, label="P1", c="r")
+    plt.scatter(-18, 0, label="P2", c="r")
+    plt.scatter(classification_request.content["objective_values"][0], 0, label="Present Position")
+    plt.xlabel("Objective Function")
+    plt.ylabel("Value")
+    plt.title("Objective Function F1")
+
+    plt.show()
+
+    # Plotting F2!
+    plt.scatter(x2, y, label="Range")
+    plt.scatter(-6, 0, label="P1", c="r")
+    plt.scatter(-9, 0, label="P2", c="r")
+    plt.scatter(classification_request.content["objective_values"][1], 0, label="Present Position")
+    plt.xlabel("Objective Function")
+    plt.ylabel("Value")
+    plt.title("Objective Function F2")
+
+    plt.show()
+
+    response = {"classifications": ["<", "="], "levels": [-7.133133133133133, 0], "number_of_solutions": 1}
+    classification_request.response = response
+
+    save_request, plot_request = method.iterate(classification_request)
+
+    print(save_request.content["objectives"])
+    print(save_request.content["solutions"])
+
+    # Plotting F1!
+    plt.scatter(x1, y, label="Range")
+    plt.scatter(-15, 0, label="P1", c="r")
+    plt.scatter(-18, 0, label="P2", c="r")
+    plt.scatter(save_request.content["objectives"][0][0], 0, label="Present Position")
+    plt.xlabel("Objective Function")
+    plt.ylabel("Value")
+    plt.title("Objective Function F1")
+
+    plt.show()
+
+    # Plotting F2!
+    plt.scatter(x2, y, label="Range")
+    plt.scatter(-6, 0, label="P1", c="r")
+    plt.scatter(-9, 0, label="P2", c="r")
+    plt.scatter(save_request.content["objectives"][0][1], 0, label="Present Position")
+    plt.xlabel("Objective Function")
+    plt.ylabel("Value")
+    plt.title("Objective Function F2")
+
+    plt.show()
+
+"""
